//+------------------------------------------------------------------+
//|                         Market_Statistics_v6_2_CPU_intensive.mq4 |
//|                                     Copyright  2010, Akif TOKUZ |
//|                                             akifusenet@gmail.com |
//| Volume histogram implementation is based on Vadim Shumilov       |
//| (DrShumiloff)'s VolumesHist2.3 indicator. Other concepts are     |
//| inspired from Jperl's `Trading With Market Statistics` thread on |
//| Traders Laboratory.                                              |
//|                                                                  |
//| History:                                                         |
//| 17.03.2010: v6.2=>Bug-fix of resetting at week start             |
//| 04.03.2010: v6.1=>Record MTF calculated stat values for every bar|
//| 17.01.2010: v5.0=>Sliding window for days back instead of fixed  |
//|               =>starting date(calculate starting from x day      |
//|               =>back, starting date is updated automatically in  |
//|               =>real-time)                                       |
//| 17.01.2010: v4.2=>Small Bugfix [zero division possibility ]      |
//|               =>When time period change call deinit and init     |
//| 09.09.2009: v4.1=>Small bugfix [delete startDate correct prefix] |
//| 09.09.2009: v4=>Instead of fixed start date now we have a        |
//|               =>relative startdate like 2 days back at 22:00     |
//| 02.09.2009: v3=>Calculation done only at the start of a new bar  |
//|               =>implementation is corrected                      |
//|               =>histogram defaulted 50 bar sd3show made true     |
//| 10.08.2009: v2=>End date added.SD2Show defaulted to true         |
//|               =>Show/disable options added for every line        |
//| 06.08.2009: v1=>Initial release                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Akif TOKUZ"
#property link      "akifusenet@gmail.com"

#property indicator_chart_window

#property indicator_buffers 8

#property indicator_color1 Red       //PVP
#property indicator_width1 2

#property indicator_color2 Aqua      //VWAP
#property indicator_width2 2

#property indicator_color3 Lime      //SD1Pos
#property indicator_width3 1
#property indicator_style3 2

#property indicator_color4 Chocolate //SD1Neg
#property indicator_width4 1
#property indicator_style4 2

#property indicator_color5 Lime      //SD2Pos
#property indicator_width5 1
#property indicator_style5 2

#property indicator_color6 Chocolate //SD2Neg
#property indicator_width6 1
#property indicator_style6 2

#property indicator_color7 Lime      //SD3Pos
#property indicator_width7 1
#property indicator_style7 2

#property indicator_color8 Chocolate //SD3Neg
#property indicator_width8 1
#property indicator_style8 2



//---- input parameters
extern string  calculationTimePeriod="1=M1 5=M5...1440=D1 10080=W1 43200=MN1";
extern int     calculateUsing = 15;
extern int     startCalculatingBackFromMinutes = 1440;
extern double  slidingWindowWidthInDays=1;
extern bool    Show_SD1 = true;
extern bool    Show_SD2 = true;
extern bool    Show_SD3 = true;
extern bool    Show_PVP = true;
extern bool    Show_VWAP = true;


//---- buffers
double PVP[];
double VWAP[];
double SD1Pos[];
double SD1Neg[];
double SD2Pos[];
double SD2Neg[];
double SD3Pos[];
double SD3Neg[];

int totalBarCount=-1;


double PVP_temp;
double VWAP_temp;
double SD1Pos_temp;
double SD1Neg_temp;
double SD2Pos_temp;
double SD2Neg_temp;
double SD3Pos_temp;
double SD3Neg_temp;


int init()
{
   
//---- indicators
   IndicatorBuffers(8);
   
   if (Show_PVP==true) SetIndexStyle(0,DRAW_LINE);
   else  SetIndexStyle(0,DRAW_NONE);
   SetIndexLabel(0,"PVP");      
   SetIndexBuffer(0,PVP);
   
   if (Show_VWAP==true) SetIndexStyle(1,DRAW_LINE);
   else  SetIndexStyle(1,DRAW_NONE);
   SetIndexLabel(1,"VWAP");      
   SetIndexBuffer(1,VWAP);
   
   if (Show_SD1==true) SetIndexStyle(2,DRAW_LINE);
   else  SetIndexStyle(2,DRAW_NONE);
   SetIndexLabel(2,"SD1Pos");      
   SetIndexBuffer(2,SD1Pos);
   
   if (Show_SD1==true) SetIndexStyle(3,DRAW_LINE);
   else  SetIndexStyle(3,DRAW_NONE);
   SetIndexLabel(3,"SD1Neg");      
   SetIndexBuffer(3,SD1Neg);
   
   if (Show_SD2==true) SetIndexStyle(4,DRAW_LINE);
   else  SetIndexStyle(4,DRAW_NONE);
   SetIndexLabel(4,"SD2Pos");      
   SetIndexBuffer(4,SD2Pos);
   
   if (Show_SD2==true) SetIndexStyle(5,DRAW_LINE);
   else  SetIndexStyle(5,DRAW_NONE);
   SetIndexLabel(5,"SD2Neg");      
   SetIndexBuffer(5,SD2Neg);
   
   if (Show_SD3==true) SetIndexStyle(6,DRAW_LINE);
   else  SetIndexStyle(6,DRAW_NONE);
   SetIndexLabel(6,"SD3Pos");      
   SetIndexBuffer(6,SD3Pos);
   
   if (Show_SD3==true) SetIndexStyle(7,DRAW_LINE);
   else  SetIndexStyle(7,DRAW_NONE);
   SetIndexLabel(7,"SD3Neg");      
   SetIndexBuffer(7,SD3Neg);
   
   string short_name="Market_Statistics";
   IndicatorShortName(short_name);
    
   return(0);
}


// Finds the endbar number for the given enddate
int findEndBarIndex(datetime enddate)
{
   for (int i=iBars(NULL,calculateUsing)-1; i>=0; i--)
   {      
      if (iTime(NULL,calculateUsing,i)>=enddate) return (i);
   }
   return( 0);
} 



int MSv3_calculate(datetime  EndDate)
{
      double Hist[]; // drawn specifically
      int      items;         // numbers of items inside volume histogram
      int      StartBar = 0; // Shows the starting bar for given date
      int      EndBar = 0; // Shows the starting bar for given date
      int      Length;
      double   SD;            // standart deviation


      double TotalVolume=0;
      double TotalPV=0;
      int n;
      

      PVP_temp=EMPTY_VALUE;
      VWAP_temp=EMPTY_VALUE;
      SD1Pos_temp=EMPTY_VALUE;
      SD1Neg_temp=EMPTY_VALUE;
      SD2Pos_temp=EMPTY_VALUE;
      SD2Neg_temp=EMPTY_VALUE;
      SD3Pos_temp=EMPTY_VALUE;
      SD3Neg_temp=EMPTY_VALUE;
             
      Length = slidingWindowWidthInDays*24*60/calculateUsing;
      EndBar=findEndBarIndex(EndDate);
      StartBar=EndBar + Length;
      
      double max = iHigh(NULL,calculateUsing,iHighest( NULL , calculateUsing, MODE_HIGH, Length, EndBar));
      double min = iLow(NULL,calculateUsing,iLowest( NULL , calculateUsing, MODE_LOW, Length, EndBar));
      items = MathRound((max - min) / Point);

      ArrayResize(Hist, items);
      ArrayInitialize(Hist, 0);

      TotalVolume=0;
      TotalPV=0;
      //Print("StartBar:",TimeToStr(iTime(NULL,calculateUsing,StartBar))," EndBar:",TimeToStr(iTime(NULL,calculateUsing,EndBar)));
      for (int i = StartBar; i >= EndBar; i--)
      {         
      
         double t1 = iLow(NULL,calculateUsing,i), t2 = iOpen(NULL,calculateUsing,i), t3 = iClose(NULL,calculateUsing,i), t4 = iHigh(NULL,calculateUsing,i);
         if (t2 > t3) {t3 = iOpen(NULL,calculateUsing,i); t2 = iClose(NULL,calculateUsing,i);}
         double totalRange = 2*(t4 - t1) - t3 + t2;         

         if (totalRange != 0.0)
         {
            for (double Price_i = t1; Price_i <= t4; Price_i += Point)
            {
               n = MathRound((Price_i - min) / Point);

               if (t1 <= Price_i && Price_i <  t2)
               {
                  Hist[n] += MathRound(iVolume(NULL,calculateUsing,i)*2*(t2-t1)/totalRange);
               }
               if (t2 <= Price_i && Price_i <= t3)
               {
                  Hist[n] += MathRound(iVolume(NULL,calculateUsing,i)*(t3-t2)/totalRange);
               }
               if (t3 <  Price_i && Price_i <= t4)
               {
                  Hist[n] += MathRound(iVolume(NULL,calculateUsing,i)*2*(t4-t3)/totalRange);
               }
            }//for
         }else
         {
            // Check if all values are equal to each other
            n = MathRound((t3 - min) / Point);
            Hist[n] += iVolume(NULL,calculateUsing,i);                     
         }//if


         // use H+L+C/3 as average price
         TotalPV+=iVolume(NULL,calculateUsing,i)*((iLow(NULL,calculateUsing,i)+iHigh(NULL,calculateUsing,i)+iClose(NULL,calculateUsing,i))/3);
         TotalVolume+=iVolume(NULL,calculateUsing,i);                          
      
      }//for BARS BACK
      
      if (TotalVolume!=0)
      {
            PVP_temp=min+ArrayMaximum(Hist)*Point;        
            VWAP_temp=TotalPV/TotalVolume;

            SD=0;         
            for (int k = StartBar; k >= EndBar; k--)
            {
               double avg=(iHigh(NULL,calculateUsing,k)+iClose(NULL,calculateUsing,k)+iLow(NULL,calculateUsing,k))/3;
               double diff=avg-VWAP_temp;
               SD+=(iVolume(NULL,calculateUsing,k)/TotalVolume)*(diff*diff);         
             }
             SD=MathSqrt(SD);
             SD1Pos_temp=VWAP_temp+SD;
             SD1Neg_temp=VWAP_temp-SD;
             SD2Pos_temp=SD1Pos_temp+SD;
             SD2Neg_temp=SD1Neg_temp-SD;
             SD3Pos_temp=SD2Pos_temp+SD;
             SD3Neg_temp=SD2Neg_temp-SD;
      
      }
   
   return(0);
}
  

  
int start()
{


   int i,counted_bars=IndicatorCounted();


   // Initialization (RUNS ONLY ONCE)
   if (totalBarCount == -1)
   {   
      i=1;
      while(i*Period()<startCalculatingBackFromMinutes)
      {
         MSv3_calculate(Time[i-1]);
         PVP[i]=PVP_temp;        
         VWAP[i]=VWAP_temp; 
         SD1Pos[i]=SD1Pos_temp;
         SD1Neg[i]=SD1Neg_temp;
         SD2Pos[i]=SD2Pos_temp;
         SD2Neg[i]=SD2Neg_temp;
         SD3Pos[i]=SD3Pos_temp;
         SD3Neg[i]=SD3Neg_temp;
         i++;
     }
     totalBarCount = Bars;
   }

   if (totalBarCount != Bars)
   {
      MSv3_calculate(Time[0]);
      PVP[1]=PVP_temp;        
      VWAP[1]=VWAP_temp; 
      SD1Pos[1]=SD1Pos_temp;
      SD1Neg[1]=SD1Neg_temp;
      SD2Pos[1]=SD2Pos_temp;
      SD2Neg[1]=SD2Neg_temp;
      SD3Pos[1]=SD3Pos_temp;
      SD3Neg[1]=SD3Neg_temp;
      totalBarCount = Bars;
   }
   
   return(0);

}

int deinit()
{
   return(0);
}